<?
/**
 * Free Wallpaper Script
 *
 * Free Wallpaper Script by Vepa Halliyev is licensed under a Creative Commons Attribution-Share Alike 3.0 License.
 *
 * @package		Free Wallpaper Script
 * @author		Vepa Halliyev
 * @copyright	Copyright (c) 2009, Vepa Halliyev, veppa.com.
 * @license		http://www.veppa.com/free-wallpaper-script/
 * @link		http://www.veppa.com/free-wallpaper-script/
 * @since		Version 1.0
 * @filesource
 */
class AdminController extends Controller
{
	private static $num_records = 30; // records per page

	public function __construct()
	{
		// layout
		$this->setLayout('backend');

		// check if user ia admin
		AuthUser::isLoggedIn(true);

		$this->assignToLayout('sidemenu' , $this->_sidemenu());
	}

	function index()
	{
		// get pending wallpapers
		if(AuthUser::hasPermission('moderator'))
		{
			// pending wallpapers
			$num_pending_wallpapers = Wallpaper::countFrom('Wallpaper','public=?',array(0));
		}

		if(AuthUser::hasPermission('admin'))
		{
			// pending users
			$num_pending_users = User::countFrom('User','level=?',array(User::PERMISSION_PENDING));
		}

		// all users can see pending comments
		$num_pending_comments = Comment::countFrom('Comment','cm_active=?',array(0));
		$total_comments = Comment::countFrom('Comment');
		$total_wallpapers = Wallpaper::countFrom('Wallpaper');
		$total_users = User::countFrom('User');

		$this->display('admin/index',array(
			'num_pending_wallpapers' => $num_pending_wallpapers,
			'num_pending_users' => $num_pending_users,
			'num_pending_comments' => $num_pending_comments,
			'total_comments' => $total_comments,
			'total_wallpapers' => $total_wallpapers,
			'total_users' => $total_users
		));
	}


	private function _sidemenu()
	{
		$menu[] = $this->_menuItem(__('Dashboard'),'admin/','user');

		$menu[] = $this->_menuItem(__('Wallpapers'),'admin/wallpapers/','user',array(
		$this->_menuItem(__('Add'),'admin/wallpapers/add/','user'),
		$this->_menuItem(__('Pending'),'admin/wallpapers/pending/','moderator')
		));

		$menu[] = $this->_menuItem(__('Comments'),'admin/comments/','moderator',array(
		$this->_menuItem(__('Pending'),'admin/comments/pending/','moderator')
		));

		$menu[] = $this->_menuItem(__('Cateogries'),'admin/categories/','moderator',array(
		$this->_menuItem(__('Add'),'admin/categories/edit/','moderator')
		));

		$menu[] = $this->_menuItem(__('Tags'),'admin/tags/','moderator');

		$menu[] = $this->_menuItem(__('Users'),'admin/users/','admin',array(
		$this->_menuItem(__('Add'),'admin/users/edit/','admin'),
		$this->_menuItem(__('Pending'),'admin/users/pending/','admin')
		));

		$menu[] = $this->_menuItem(__('Settings'),'admin/settings/','admin');

		return $menu;
	}

	private function _menuItem($name, $url, $permission, $submenu=array())
	{
		return array('name' 		=>	$name,
					'url'			=>	$url,
					'permission'	=>	$permission,
					'submenu'		=>	$submenu	);
	}

	public static function renderMenu($menuItems = array(),$menu_selected = '')
	{

		$return = '';
		if($menuItems)
		{
			foreach($menuItems as $menu)
			{
				// chek permission
				if(AuthUser::hasPermission($menu['permission']))
				{
					$class = ($menu['url'] == $menu_selected?' class="menu_selected"':'');

					$return .= '<li'.$class.'><a href="'.get_url($menu['url']).'">'.$menu['name'].'</a>';
					$return .= self::renderMenu($menu['submenu']);
					$return .= '</li>';
				}
			}
			if($return)
			{
				$return = '<ul>'.$return.'</ul>';
			}
		}

		return $return;
	}


	function wallpapers()
	{
		$args = func_get_args();

		$mode = $args[0];

		switch($mode)
		{
			case 'add':
			case 'edit':
			case 'delete':
			case 'activate':
				return $this->{'wallpapers'.ucfirst($mode)}($args[1]);
				break;
			case 'pending':
				// check user permission. only admin and moderator can view pending wallpapers
				AuthUser::hasPermission(User::PERMISSION_MODERATOR,0,true);

				// page number
				$page = $args[1];
				$paginator_pattern = 'admin/wallpapers/pending/{page}/';

				// list anly pending wallpapers
				$public = 0;
				break;
			default:
			case 'page':
				// page number
				$page = $args[1];
				$paginator_pattern = 'admin/wallpapers/page/{page}/';

				// list anly pending wallpapers
				$public = 1;
				break;
		}

		// set page number to 1 if not set
		if(!$page)
		{
			$page = 1;
		}

		// get wallpapers
		$st = ($page-1) * self::$num_records;
		$wallpapers = Wallpaper::findAllFrom('Wallpaper','public=? ORDER BY id DESC LIMIT '.$st.','.self::$num_records,array($public));

		// populate paginator
		$total_wallpapers = Wallpaper::countFrom('Wallpaper','public=?',array($public));
		$total_pages = ceil($total_wallpapers / self::$num_records);
		$paginator = Paginator::render($page,$total_pages,get_url($paginator_pattern));


		$this->display('admin/wallpapers',array(
			'wallpapers' =>	$wallpapers,
			'paginator'	=>	$paginator,
			'public'	=>	$public
		));
	}

	function wallpapersEdit()
	{
		$this->_hasPermissionAjax(User::PERMISSION_USER,AuthUser::$user->id,true);
		
		$id = $_POST['id'];

		if(!$id)
		{
			exit('Wallpaper not found.');
		}

		$wallpaper = Wallpaper::findByIdFrom('Wallpaper',$id,'id');
		if( !$wallpaper )
		{
			exit('Wallpaper not found.');
		}

		$categories = Category::getCategories();
		
		// save changes if submitted
		$this->_wallpapersEdit($categories);

		$this->setLayout(false);
		$this->display('admin/wallpapersEdit',array(
			'wallpaper'	=>	$wallpaper,
			'categories'	=>	$categories,
			'title'	=>	__('Edit wallpaper'),
		));
	}

	private function _wallpapersEdit($categories=array())
	{
		if($_POST['action']=='update_wallpaper')
		{

			$id = $_POST['id'];

			parse_str($_POST['data'],$post);
			$_POST = $post;

			// mix tags with categories
			$_POST['tags'] = Tag::mixWithCategories($_POST['tags'],$categories,$_POST['categories']);			
			
			$rules['name'] = 'trim|required|strip_tags|xss_clean';
			$rules['tags'] = 'trim|strip_tags|xss_clean';
			$rules['artist'] = 'trim|strip_tags|xss_clean';
			$rules['site'] = 'trim|strip_tags|xss_clean';
			$rules['description'] = 'trim|strip_tags|xss_clean';
			$fields['name'] = __('Name');

			$this->validation()->set_rules($rules);
			$this->validation()->set_fields($fields);

			if($this->validation()->run())
			{
				//print_r($post);
				$post = $_POST;
				
				$fields = 'name,artist,site,description,tags';
				$wallpaper = new Wallpaper(Wallpaper::filterCols($post,$fields));
				$wallpaper->id = $id;
				$wallpaper->save('id');
					
				// select wallpaper and create table cell to display
				$w = Wallpaper::findByIdFrom('Wallpaper',$id,'id');

				$this->setLayout(false);

				echo "ok{SEP}";
				$this->display('admin/snippetWallpaperCell',compact('w'));
			}
			else
			{
				echo strip_tags(str_replace('</',' ',$this->validation()->messages()));
			}
			exit;
		}
	}


	function wallpapersActivate()
	{
		// only moderator can activa uploaded wallpaper
		$this->_hasPermissionAjax(User::PERMISSION_MODERATOR,0,true);
		
		$id = $_POST['id'];
		if(!$id)
		{
			exit(__('Wallpaper not found.'));
		}
		$wallpaper = Wallpaper::findByIdFrom('Wallpaper',$id,'id');
		if(!$wallpaper)
		{
			exit(__('Wallpaper not found.'));
		}

		$wallpaper->public = 1;
		$wallpaper->save('id');

		exit('ok');
	}
	
	private function _hasPermissionAjax($permission,$user_id,$redirect)
	{
		$result = AuthUser::hasPermission($permission,$user_id,false);
		if(!$result && $redirect)
		{
			exit(__('No permission to perform this action.'));
		}
		
		return $result;
	}


	function wallpapersAdd()
	{
		$categories = Category::getCategories();
		
		// save changes if submitted
		$this->_wallpapersAdd($categories);

		$this->display('admin/wallpapersEdit',array(
			'title'	=>	__('Add wallpaper'),
			'categories'	=>	$categories,
			'add'	=>	true,
		));
	}


	private function _wallpapersAdd($categories=array())
	{
		if(get_request_method()=='POST')
		{
			// mix tags with categories
			$_POST['tags'] = Tag::mixWithCategories($_POST['tags'],$categories,$_POST['categories']);
			
			$rules['name'] 	= 'trim|required|strip_tags|xss_clean';
			$rules['tags'] 	= 'trim|required|strip_tags|xss_clean';
			$rules['artist'] = 'trim|strip_tags|xss_clean';
			$rules['site'] 	= 'trim|strip_tags|xss_clean';
			$rules['description'] = 'trim|strip_tags|xss_clean';
			$fields['name'] = __('Name');

			$this->validation()->set_rules($rules);
			$this->validation()->set_fields($fields);

			if($this->validation()->run())
			{
				if(false === $uploaded_file = Wallpaper::upload('img'))
				{
					$this->validation()->set_error(Wallpaper::displayErrors('',' '));
					return false;
				}

				// image uploaded and resized. add record to database

				$post = $_POST;

				$post['img'] = $uploaded_file['name'];
				$post['size'] = $uploaded_file['size'];

				$fields = array('name', 'artist','site', 'description','img','size','tags');
				$wallpaper = new Wallpaper( Wallpaper::filterCols($post,$fields) );

				// if user is moderator make wallpaper automaticly public.
				if(AuthUser::hasPermission('moderator'))
				{
					$wallpaper->public = 1;
				}

				if(!$wallpaper->save('id'))
				{
					// error saving. delete image
					Wallpaper::deleteWallpaperImg($wallpaper->img);

					$this->validation()->set_error(__('Error adding uploded wallpaper.'),$field);
					return false;
				}

				$id = $wallpaper->id;
					
				// wallpaper added. redirect to add page to avoid double insertion
				Flash::set('success',__('Wallpaper uploaded.'));
				redirect(get_url('admin/wallpapers/add/'));

				return true;
			}
		}
	}
	
	function wallpapersDelete()
	{
		$this->_hasPermissionAjax(User::PERMISSION_USER,AuthUser::$user->id,true);
		
		$id = $_POST['id'];
		if(!$id)
		{
			exit('Wallpaper not found.');
		}

		// get wallpaper from db to delete its source
		$wallpaper = Wallpaper::findByIdFrom('Wallpaper',$id,'id');
		if(!$wallpaper)
		{
			exit('Wallpaper not found.');
		}

		// delete form db and image files
		$wallpaper->delete('id');

		exit('ok');
	}




	function categories()
	{
		AuthUser::hasPermission(User::PERMISSION_MODERATOR,0,true);
		
		$args = func_get_args();

		$mode = $args[0];

		switch($mode)
		{
			case 'add':
			case 'edit':
			case 'delete':
			case 'activate':
				return $this->{'categories'.ucfirst($mode)}($args[1],$args[2]);
				break;
			default:
				$parent_id = $mode>0?$mode:0;
				break;
		}

		// get categories
		$categories = Category::findAllFrom('Category','1=1 ORDER BY name');


		$this->display('admin/categories',array(
			'categories' => $categories,
			'parent_id' => $parent_id,
		));
	}


	function categoriesEdit($id=0,$parent_id=0)
	{
		AuthUser::hasPermission(User::PERMISSION_MODERATOR,0,true);
		
		// save changes if submitted
		$this->_categoriesEdit();

		if($id)
		{
			$category = Category::findByIdFrom('Category',$id,'id');
			if( !$category )
			{
				redirect(get_url('admin/categories/'));
			}
			$title = __('Edit category');
			$add = false;
		}
		else
		{
			$title = __('Add category');
			$add = true;
		}
			
		$this->display('admin/categoriesEdit',array(
			'category'	=>	$category,
			'title'	=>	$title,
			'add'	=>	$add,
		));
	}


	function categoriesDelete($id=0)
	{
		$this->_hasPermissionAjax(User::PERMISSION_MODERATOR,0,true);
		
		// save changes if submitted
		$this->_categoriesDelete();

		if(!$id)
		{
			redirect(get_url('admin/cateogries'));
		}
			
		$category = Category::findByIdFrom('Category',$id,'id');
		if( !$category )
		{
			redirect(get_url('admin/categories/'));
		}


		// get subcategories tree


		$this->display('admin/categoriesDelete',array(
			'category'	=>	$category,
			'subcategory_tree'	=>	$subcategory_tree,
		));
	}

	private function _categoriesEdit()
	{
		if(get_request_method()=='POST')
		{
			$rules['name'] = 'trim|required|strip_tags|xss_clean';
			$fields['name'] = __('Name');

			$this->validation()->set_rules($rules);
			$this->validation()->set_fields($fields);

			if($this->validation()->run())
			{
				$category = new Category(Category::filterCols($_POST,'name'));

				if($category->id)
				{
					$success_redirect = get_url('admin/categories/');
				}
				else
				{
					$success_redirect = get_url('admin/categories/edit/');
				}

				if($category->save('id'))
				{
					Flash::set('success',__('Category saved.'));
					redirect($success_redirect);
				}
				else
				{
					$this->validation()->set_error(__('Error saving category'));
				}
			}
		}
	}


	private function _categoriesDelete()
	{
		if(get_request_method()=='POST')
		{
			$category_id = $_POST['id'];
			if(!$category_id)
			{
				$this->validation()->set_error(__('Category not found.'));
				return false;
			}


			$category = Category::findByIdFrom('Category',$category_id,'id');
			if(!$category)
			{
				$this->validation()->set_error(__('Category not found.'));
				return false;
			}

			// delete category
			if($category->delete('id'))
			{
				Flash::set('success',__('Category deleted.'));
				redirect(get_url('admin/categories/'));
			}
			else
			{
				$this->validation()->set_error(__('Error deleting category. Please try again.'));
			}
		}
	}


	function categoriesActivate()
	{
		
		$this->_hasPermissionAjax(User::PERMISSION_MODERATOR,0,true);
		
		$id = $_POST['id'];
		if(!$id)
		{
			exit(__('Category not found.'));
		}
		$category = Category::findByIdFrom('Category',$id,'id');
		if(!$category)
		{
			exit(__('Category not found.'));
		}

		if($category->c_active)
		{
			$category->c_active = 0;
			$category->save('id');
			exit(__('no'));
		}
		else
		{
			$category->c_active = 1;
			$category->save('id');
			exit(__('yes'));
		}
	}

	function tags()
	{
		AuthUser::hasPermission(User::PERMISSION_MODERATOR,0,true);
		
		$args = func_get_args();

		$mode = $args[0];

		switch($mode)
		{
			case 'delete':
				return $this->{'tags'.ucfirst($mode)}($args[1]);
				break;
			default:
				$parent_id = $mode>0?$mode:0;
				break;
		}

		$this->display('admin/tags',array(
			'tags' => Tag::getAllTags(0),
		));
	}

	function tagsDelete()
	{
		$this->_hasPermissionAjax(User::PERMISSION_MODERATOR,0,true);
		
		$tag = $_POST['tag'];

		$t = Tag::findByIdFrom('Tag', $tag, 't_name');
		if($t)
		{
			$t->delete('t_id');
		}
		exit('ok');
	}


	function users()
	{
		// check user permission. only admin and moderator can view pending wallpapers
		AuthUser::hasPermission(User::PERMISSION_ADMIN,0,true);

		$args = func_get_args();

		$mode = $args[0];

		switch($mode)
		{
			case 'add':
			case 'edit':
			case 'delete':
			case 'verify':
				return $this->{'users'.ucfirst($mode)}($args[1]);
				break;
			case 'pending':
				// page number
				$page = $args[1];
				$paginator_pattern = 'admin/users/pending/{page}/';

				// list only pending users
				$activation = "activation <> '0'";
				$pending = true;
				break;
			default:
			case 'page':
				// page number
				$page = $args[1];
				$paginator_pattern = 'admin/users/page/{page}/';

				// list only activated users
				$activation = "activation = '0'";
				$pending = false;
				break;
		}

		// set page number to 1 if not set
		if(!$page)
		{
			$page = 1;
		}

		// get wallpapers
		$st = ($page-1) * self::$num_records;
		$users = User::findAllFrom('User',$activation.' ORDER BY username ASC LIMIT '.$st.','.self::$num_records,array());

		// populate paginator
		$total_users = User::countFrom('User',$activation,array());
		$total_pages = ceil($total_users / self::$num_records);
		$paginator = Paginator::render($page,$total_pages,get_url($paginator_pattern));


		$this->display('admin/users',array(
			'users' =>	$users,
			'paginator'	=>	$paginator,
			'pending'	=>	$pending
		));
	}

	function usersVerify()
	{
		$this->_hasPermissionAjax(User::PERMISSION_ADMIN,0,true);
		
		$id = $_POST['id'];
		if(!$id)
		{
			exit(__('User not found.'));
		}
		$user = User::findByIdFrom('User',$id,'id');
		if(!$user)
		{
			exit(__('User not found.'));
		}

		$user->activation = 0;
		$user->save('id');
		exit('ok');
	}

	function usersDelete()
	{
		$this->_hasPermissionAjax(User::PERMISSION_ADMIN,0,true);
		
		$id = $_POST['id'];
		if(!$id)
		{
			exit(__('User not found.'));
		}
		$user = User::findByIdFrom('User',$id,'id');
		if(!$user)
		{
			exit(__('User not found.'));
		}

		$user->delete('id');
		exit('ok');
	}



	function usersEdit($id=0)
	{
		AuthUser::hasPermission(User::PERMISSION_ADMIN,0,true);
		
		// save changes if submitted
		// return submitted user object
		$user = $this->_usersEdit();

		if($id)
		{
			// if no user data submitted then get user from db
			if(!$user)
			{
				$user = User::findByIdFrom('User',$id,'id');
				if( !$user )
				{
					redirect(get_url('admin/users/'));
				}
			}
			$title = __('Edit user');
			$add = false;
		}
		else
		{
			$title = __('Add user');
			$add = true;
		}
			
		$this->display('admin/usersEdit',array(
			'user'	=>	$user,
			'title'	=>	$title,
			'add'	=>	$add,
		));
	}
	
	
	function settings()
	{
		AuthUser::hasPermission(User::PERMISSION_ADMIN,0,true);
		
		$this->_settings();
		
		// get editable settings 
		$settings = Config::findAllFrom('Config','is_Editable!=0 ORDER BY name',array());
			
		$this->display('admin/settings',array(
			'settings'	=>	$settings
		));
	}
	
	
	private function _settings()
	{
		if(get_request_method()=='POST')
		{
			foreach($_POST as $k=>$v)
			{
				$data['val'] = $v;
				Config::update('Config',$data,'name=?',array($k)); 
			}
			$this->validation()->set_success(__('Settings updated.'));
		}
	}


	/**
	 * Add new user or update existing user
	 * @return User object|redirect
	 */
	private function _usersEdit()
	{
		if(get_request_method()=='POST')
		{
			$id = $_POST['id'];


			$rules['username'] = 'trim|required|strip_tags|xss_clean';
			$rules['email'] = 'trim|required|strip_tags|xss_clean|valid_email';
			$rules['web'] = 'trim|strip_tags|xss_clean';
			$rules['level'] = 'intval';

			if(strlen($_POST['password']))
			{
				$rules['password'] = 'min_length[4]|max_length[32]|matches[password_repeat]';
			}
			elseif(!$id)
			{
				// adding new user ask for password
				$rules['password'] = 'required|min_length[4]|max_length[32]|matches[password_repeat]';
			}

			$this->validation()->set_rules($rules);
			$this->validation()->set_fields($fields);

			if($this->validation()->run())
			{
				//print_r($post);
					
				$fields = array('id', 'username', 'email','web', 'level');
				$user = new User(User::filterCols($_POST,$fields));
				
				if(strlen($_POST['password']))
				{
					$user->password = md5($_POST['password']);
				}

				if($user->save('id'))
				{
					Flash::set('success',__('User saved.'));
					redirect(get_url('admin/users/'));
				}
				else
				{
					$this->validation()->set_error(__('Error saving user.'));
				}
			}

			return new User($_POST);
		}
	}


	/**
	 * update g_tags value for wallpapers using tag relation
	 * @return unknown_type
	 */
	function updateAllTags($ajax=0,$page=0)
	{
		//
		if($page)
		{
			$num = 100;
			$st = ($page-1)*$num;
			$wallpapers = Wallpaper::findAllFrom('Wallpaper',"1=1 LIMIT $st,$num",array(),MAIN_DB,'id');
			if(!$wallpapers)
			{
				page_not_found();
			}
			// get tags from relation table
			Tag::relatedTags($wallpapers,0,'id');

			// update tags str
			foreach($wallpapers as $w)
			{
				if(is_array($w->tags))
				{
					foreach($w->tags as $t)
					{
						$w->tags .= $t->t_name.',';
					}
				}
				$msg .= $w->id.':'.$w->tags.'<br/>';
				$w->save('id');
			}
			//print_r($wallpapers);

			$msg = __('updated {num} records',array(
				'{num}'=>count($wallpapers)
			)).$msg;
		}

		if($ajax)
		{
			exit($msg.'<br/>');
		}
		else
		{
			$this->display('admin/updateAllTags',array(
				'msg' => $msg
			));
		}
	}




	function comments()
	{
		AuthUser::hasPermission(User::PERMISSION_MODERATOR,0,true);
		
		$args = func_get_args();

		$mode = $args[0];

		switch($mode)
		{
			case 'edit':
			case 'delete':
				return $this->{'comments'.ucfirst($mode)}($args[1]);
				break;
			case 'activate':
				return $this->{'comments'.ucfirst($mode)}($args[1]);
				break;
			case 'pending':

				// page number
				$page = $args[1];
				$paginator_pattern = 'admin/comments/pending/{page}/';

				// list only pending comments
				$public = 0;
				break;
			default:
			case 'page':
				// page number
				$page = $args[1];
				$paginator_pattern = 'admin/comments/page/{page}/';

				// list anly pending wallpapers
				$public = 1;
				break;
		}

		// set page number to 1 if not set
		if(!$page)
		{
			$page = 1;
		}

		// get wallpapers
		$st = ($page-1) * self::$num_records;
		$comments = Comment::findAllFrom('Comment','cm_active=? ORDER BY cm_id DESC LIMIT '.$st.','.self::$num_records,array($public));

		// populate paginator
		$total = Comment::countFrom('Comment','cm_active=?',array($public));
		$total_pages = ceil($total / self::$num_records);
		$paginator = Paginator::render($page,$total_pages,get_url($paginator_pattern));

		// append wallpapers
		Comment::appendObject($comments,'cm_wid','Wallpaper','id','wallpaper',MAIN_DB,'id,name,img',true);
		Comment::appendObject($comments,'cm_added_by','User','id','user',MAIN_DB,'id,username',true);

		$this->display('admin/comments',array(
			'comments' =>	$comments,
			'public' =>	$public,
			'paginator'	=>	$paginator
		));
	}

	function commentsEdit()
	{
		AuthUser::hasPermission(User::PERMISSION_MODERATOR,0,true);
		
		$id = $_POST['cm_id'];

		if(!$id)
		{
			exit('Comment not found.');
		}

		$comment = Comment::findByIdFrom('Comment',$id,'cm_id');
		if( !$comment )
		{
			exit('Comment not found.');
		}

		if($_POST['action']=='update_comment')
		{
			$rules['cm_body'] = 'trim|required|strip_tags|xss_clean';

			if(!$comment->cm_added_by)
			{
				$rules['cm_name'] = 'trim|required|strip_tags|xss_clean';
				$rules['cm_website'] = 'trim|strip_tags|xss_clean';
			}

			$fields['cm_name'] = __('Name');
			$fields['cm_website'] = __('Website');
			$fields['cm_body'] = __('Body');

			$this->validation()->set_rules($rules);
			$this->validation()->set_fields($fields);

			if($this->validation()->run())
			{
				if(!$comment->cm_added_by)
				{
					$comment->cm_name = $_POST['cm_name'];
					$comment->cm_website = $_POST['cm_website'];
				}
				$comment->cm_body = $_POST['cm_body'];

				$comment->save('cm_id');
					

				Comment::appendObject($comment,'cm_added_by','User','id','user',MAIN_DB,'id,username',true);

				$this->setLayout(false);
				echo "ok{SEP}";
				$this->display('admin/snippetCommentCell',array('c'=>$comment));
			}
			else
			{
				echo strip_tags(str_replace('</',' ',$this->validation()->messages()));
			}
		}
		else
		{
			exit('No action.');
		}
	}

	function commentsActivate()
	{
		$this->_hasPermissionAjax(User::PERMISSION_MODERATOR,0,true);
		
		$id = $_POST['cm_id'];
		if(!$id)
		{
			exit(__('Comment not found.'));
		}
		$comment = Comment::findByIdFrom('Comment',$id,'cm_id');
		if(!$comment)
		{
			exit(__('Comment not found.'));
		}

		$comment->cm_active = 1;
		$comment->save('cm_id');

		// update number of comments
		Wallpaper::updateNumComments($comment->cm_wid);

		exit('ok');
	}

	function commentsDelete()
	{
		$this->_hasPermissionAjax(User::PERMISSION_MODERATOR,0,true);
		
		$id = $_POST['cm_id'];
		if(!$id)
		{
			exit('Comment not found.');
		}

		// get wallpaper from db to delete its source
		$comment = Comment::findByIdFrom('Comment',$id,'cm_id');
		if(!$comment)
		{
			exit('Comment not found.');
		}

		// delete form db and image files
		$comment->delete('cm_id');

		exit('ok');
	}


}