<?php
/**
 * Free Wallpaper Script
 *
 * Free Wallpaper Script by Vepa Halliyev is licensed under a Creative Commons Attribution-Share Alike 3.0 License.
 *
 * @package		Free Wallpaper Script
 * @author		Vepa Halliyev
 * @copyright	Copyright (c) 2009, Vepa Halliyev, veppa.com.
 * @license		http://www.veppa.com/free-wallpaper-script/
 * @link		http://www.veppa.com/free-wallpaper-script/
 * @since		Version 1.0
 * @filesource
 */
/**
 * class Wallpaper
 *
 * @author Vepa Halliyev <veppa.com>
 * @since  0.1
 */

class Wallpaper extends Record
{
	const TABLE_NAME = 'wallpaper';
	
	private static $cols = array(
		'id'	=>	1,                              
		'img'	=>	1,   
		'name'	=>	1,                              
		'artist'	=>	1,                 
		'site'	=>	1,                
		'description'	=>	1,                              
		'tags'	=>	1,                              
		'size'	=>	1,               
		'dwn'	=>	1,                              
		'rank'	=>	1,                              
		'public'	=>	1,
		'votes'	=>	1,             
		'num_comments'	=>	1,             
		'num_all_comments'	=>	1,             
		'd1'	=>	1,                              
		'd2'	=>	1,                              
		'd3'	=>	1,                              
		'd4'	=>	1,                              
		'd5'	=>	1,                              
		'd6'	=>	1,                              
		'd7'	=>	1,                              
		'week'	=>	1,  
		'ip'	=>	1,                              
		'added_at'	=>	1,                           
		'added_by'	=>	1  
	);

	
	static $arr_sizes = array(
		'1024x768', 
		'1600x1200', 
		'2048x1536', 
		'1400x1050', 
		'1152x864', 
		'1280x960', 
		'1920x1440', 
		'1280x1024', 
		'1280x720', 
		'1680x1050', 
		'1440x900', 
		'1920x1200', 
		'1280x800' 
	);
	
	static $errors = array();

	public function __construct($data = false, $locale_db = false)
	{
		$this->setColumns(self::$cols);
		parent::__construct($data,$locale_db);
	}

	
	function afterInsert()
	{
		// add tags
		Tag::addTags($this->tags,$this->id);		
		return true;
	}
	
	function beforeInsert()
	{
		$this->ip = Controller::input()->ip_address();
		$this->added_at = REQUEST_TIME;
		$this->added_by = AuthUser::$user->id;
		
		// normalize tags 
		$this->tags = Tag::normalizeTagsStr($this->tags);
		
		return true;
	}
	
	function beforeUpdate()
	{
		// normalize and add tags 
		$this->tags = Tag::addTags($this->tags,$this->id);		
		return true;
	}
	
	function beforeDelete()
	{
		// delete related tags
		TagRelation::deleteWhere('TagRelation','r_wid=?',array($this->id));
		
		// delete related comments
		Comment::deleteWhere('Comment','cm_wid=?',array($this->id));
		
		// delete related votes
		Vote::deleteWhere('Vote','v_wid=?',array($this->id));
		
		// delete wallpaper images from disk
		self::deleteWallpaperImg($this->img);
		
		return true;
	}
	
	
	static function img($wallpaper,$size='thumb')
	{
		// size : thumb:small, mid:medium, o:original size
		if($size == 'original')
		{
			$size = 'o';
		}
		return UPLOAD_URL.'/wall/'.$size.'/'.$wallpaper->img;
	}
	
	static function imgSrc($wallpaper,$size='thumb')
	{
		// size : thumb:small, view:medium, o:original size
		if($size == 'original')
		{
			$size = 'o';
		}
		return UPLOAD_ROOT.'/wall/'.$size.'/'.$wallpaper->img;
	}
	
	
	
	static function url($wallpaper,$type='page')
	{
		// $type : page, view
		switch($type)
		{
			case 'view':
				return get_url('download/'.Inflector::slugify($wallpaper->name).'_w'.$wallpaper->id.'.html');
				break;
			case 'page':
			default:
				return get_url('wallpaper/'.Inflector::slugify($wallpaper->name).'_w'.$wallpaper->id.'.html');
				break;
		}
	}
	
	/**
	 * Upload wallpaper and return true or false on success
	 * 
	 * @uses UPLOAD_ROOT
	 * @uses FileDir::checkMakeDir
	 * @uses Upload->doUpload
	 * @uses Image->resize 
	 * 
	 * @param string $field 
	 * @param object $validation
	 * @return bool|array(name,width,height,size)
	 */
	static function upload($field)
	{
		// store images in 50 random folders to prevent 32000 file problem in one folder for linux servers.
		// TODO use first 5 letters of filename to prevent this for optimal solution
		$random_folder = rand(1,100).'/';
		$upload_path = UPLOAD_ROOT.'/wall/o/'.$random_folder;
		$th_dir = UPLOAD_ROOT.'/wall/thumb/'.$random_folder;
		$mid_dir = UPLOAD_ROOT.'/wall/mid/'.$random_folder;
		
		// create upload folder
		FileDir::checkMakeDir($upload_path);
		FileDir::checkMakeDir($th_dir);
		FileDir::checkMakeDir($mid_dir);
		
		$upload_config['upload_path'] 	= $upload_path;
		$upload_config['overwrite'] 	= false;
		$upload_config['is_image'] 	= true;
		$upload_config['allowed_types'] = 'jpg';
		$upload_config['max_size']		= '2000';
		$upload_config['max_width']  	= '3000';
		$upload_config['max_height']  	= '3000';
		
		$upload = new Upload($upload_config);
	
		if($upload->doUpload($field))
		{
			// uploaded. now resize image to thumb and medium size.
						
			$data = $upload->data();
			
			// img folder and file name to store in db
			$img = $random_folder.$data['file_name'];
			
			
			// create thumb
			$return = self::resizeImage($data['full_path'],$th_dir.$data['file_name'],'100x75',true);
			if($return)
			{
				$return['name'] = $img;
			}
			else
			{
				// error delete source
				self::deleteWallpaperImg($img);				
				return false;
			}

			
			// create medium image
			if(! self::resizeImage($data['full_path'],$mid_dir.$data['file_name'],'300x250'))
			{
				// error delete source and thumb
				self::deleteWallpaperImg($img);				
				return false;
			}
			
			
			// return file name with random folder. store in db same way 
			return $return;
		}
		else
		{
			// set error messages to validation object
			// set errors 
			return self::_uploadError($upload);
		}
	}
	
	
	/**
	 * Delete wallpaper image in all folders. original and resized images.
	 * @param $src
	 * @return unknown_type
	 */
	function deleteWallpaperImg($src)
	{
		// delete
		@unlink(UPLOAD_ROOT."/wall/thumb/".$src);
		@unlink(UPLOAD_ROOT."/wall/mid/".$src);
		@unlink(UPLOAD_ROOT."/wall/o/".$src);
		
		foreach(self::$arr_sizes as $val)
		{
			@unlink(UPLOAD_ROOT."wall/".$val.'/'.$src);
		}
	}
	
	
	static function relatedWallpapers($id,$num=15)
	{
		
		$limit = "LIMIT 0,".$num;
		
		$where = "w.public='1'";
		$where .= " AND r.r_wid=? AND r2.r_tid=r.r_tid AND w.id = r2.r_wid AND w.id!=?";
		
		$sql = "SELECT count(w.id) as t_num, w.* 
				FROM ".TABLE_PREFIX."wallpaper w,".TABLE_PREFIX."tag_relation r,".TABLE_PREFIX."tag_relation r2
				WHERE $where
				GROUP BY w.id 
				ORDER BY t_num DESC					
				$limit";
				
		return self::query($sql,array($id,$id));
	}
	
	
	
	function getMax($col='dwn')
	{
		$sql = "SELECT MAX(".$col.") as max_val FROM ".TABLE_PREFIX."wallpaper";
		$r = self::queryOne($sql,array());
		return $r->max_val;
	}
	
	
	function increaseDownloadCount()
	{
		// get last update date from conf. 
		$con_last_date_reset = Config::option('con_last_date_reset');
		
		// d1
		$day_number_col = "d".(date("w")+1);
		$today = date("Y-m-d");
		
		// if tdate is not today then save olf values and calculate weekly downloads
		if($today != $con_last_date_reset )
		{
			// save todays date
			Config::optionSet('con_last_date_reset',$today);
			
			// update weekly downloads for all tables and reset counters
			self::updateWeeklyDownloads($day_number_col);
			
			// reset today
			$this->{$day_number_col} = 0;			
		}
		
		// increase downloads
		$this->dwn = $this->dwn + 1;
		$this->{$day_number_col} = $this->{$day_number_col} + 1;
		$this->save('id');
	}
	
	
	static function updateWeeklyDownloads($day_number_col)
	{
		$sql = "UPDATE ".TABLE_PREFIX.self::TABLE_NAME."
				SET week=d1+d2+d3+d4+d5+d6+d7";
		self::query($sql);
		
		$sql = "UPDATE ".TABLE_PREFIX.self::TABLE_NAME."
				SET $day_number_col=0";
		self::query($sql);
	}
	
	
	function getByTagId($id,$order_col,$st,$num)
	{
		$limit = 'LIMIT '.$st.','.$num;
		
		$where = "w.public='1'";
		$where .= " AND r.r_tid=? AND r.r_wid=w.id";
		
		$sql = "SELECT w.*  
				FROM ".TABLE_PREFIX."wallpaper w, ".TABLE_PREFIX."tag_relation r
				WHERE $where 
				ORDER BY ".$order_col." DESC ".$limit;
		
		return Record::query($sql,array($id));
	}
	
	function getByTagIdCount($id)
	{
		$where = "w.public='1'";
		$where .= " AND r.r_tid=? AND r.r_wid=w.id";
		$sql = "SELECT count(w.id) as total  
						FROM ".TABLE_PREFIX."wallpaper w, ".TABLE_PREFIX."tag_relation r
						WHERE $where";
		$num = Record::query($sql,array($id));
		
		return $num[0]->total;
	}
	
	function updateNumComments($id)
	{
		// approved comments 
		$num_comments = Comment::countFrom('Comment','cm_wid=? AND cm_active=?',array($id,1));
		
		// all comments 
		$num_all_comments = Comment::countFrom('Comment','cm_wid=? AND cm_active=?',array($id,1));
		$data['num_comments'] = $num_comments;
		$data['num_all_comments'] = $num_all_comments;
		
		Wallpaper::update('Wallpaper',$data,'id=?',array($id));
	}
	
	// reszie source to destination. 
	// return size of original image used for image upoloads
	static function resizeImage($source,$destination,$size,$return_info = false)
	{
		// create destination folder if not exists
		FileDir::checkMakeDir(dirname($destination)); 
		
		
		list($w,$h) = explode('x',$size);

		// resize 
		$config['image_library'] = 'gd2';
		$config['source_image'] = $source;
		$config['maintain_ratio'] = false;
		$config['resize_crop'] = true;
		
		// create thumb
		$config['width'] = $w;
		$config['height'] = $h;
		$config['new_image'] = $destination;
		
		$image = new Image($config);
		
		$return['width'] = $image->orig_width;
		$return['height'] = $image->orig_height;
		$return['size'] = $return['width'].'x'.$return['height'];
		
		if($image->resize())
		{
			if($return_info)
			{
				return $return;
			}
			else
			{
				return true;
			} 
		}
		else
		{
			// set errors 
			return self::_uploadError($image);
		}
	}
	
	
	function _uploadError($obj)
	{
		// set errors 
		$error = trim($obj->displayErrors('',' '));
		//echo '_uploadError:'.$error;
		if($error)
		{
			//echo '_uploadError:'.$error;
			self::$errors[] = $error;
		}
		return false;
	}
	
	// resize current wallpaper to defined size 
	function resize($size)
	{
		// check if size allowed
		if(!in_array($size,self::$arr_sizes))
		{
			// playwood
			return false;
		}
		
		$destination = self::imgSrc($this,$size);
		$source = self::imgSrc($this,'o');
		
		$stat_img = @stat($destination);
		if($stat_img['size']==0)
		{
			// delete faulty image 70 
			@unlink($destination);
			
			// image not found resize image 
			if(!self::resizeImage($source,$destination,$size))
			{
				return false;
			}
		}
		
		return $destination;
	}
	
	function displayErrors($open = '', $close = ' ')
	{
		
		//var_dump(self::$errors );
		
		if(count(self::$errors))
		{
			$str = '';
			foreach (self::$errors as $val)
			{
				$str .= $open.$val.$close;
			}
			return $str;
		}

		return false;
	}
	
	
} // end Wallpaper class